"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _lodash = _interopRequireDefault(require("lodash"));

var _reactPlotly = _interopRequireDefault(require("react-plotly.js"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

/**
 * The OncoPrint component is used to view multiple genetic alteration events
 * through an interactive and zoomable heatmap. It is a React/Dash port of the
 * popular oncoPrint() function from the BioConductor R package.
 * Under the hood, the rendering is done using Plotly.js built upon D3.
 * Plotly's interactivity allows the user to bind clicks and hovers to genetic
 * events, allowing the user to create complex bioinformatic apps or workflows
 * that rely on crossfiltering.
 * Read more about the component here:
 * https://github.com/plotly/react-oncoprint
 */
var OncoPrint =
/*#__PURE__*/
function (_PureComponent) {
  _inherits(OncoPrint, _PureComponent);

  // Constructor
  function OncoPrint(props) {
    var _this;

    _classCallCheck(this, OncoPrint);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(OncoPrint).call(this, props));
    _this.state = {
      xStart: null,
      xEnd: null
    };
    _this.resetWindowing = _this.resetWindowing.bind(_assertThisInitialized(_assertThisInitialized(_this)));
    _this.handleChange = _lodash.default.debounce(_this.handleChange.bind(_assertThisInitialized(_assertThisInitialized(_this))), 250);
    return _this;
  } // Reset windowing to user preset on init or data change


  _createClass(OncoPrint, [{
    key: "resetWindowing",
    value: function resetWindowing(props) {
      var range = props.range;
      var xStart, xEnd;

      if (range.length === 2) {
        xStart = range[0];
        xEnd = range[1];
      } else {
        xStart = null;
        xEnd = null;
      }

      return {
        xStart: xStart,
        xEnd: xEnd
      };
    } // Handle plot events

  }, {
    key: "handleChange",
    value: function handleChange(event) {
      if (!this.props.onChange) {
        return;
      } // CLick (mousedown) or hover (mousemove)


      if (event.points) {
        var eventType;

        if (event.event.type === "mousedown") {
          eventType = 'Click';
        } else if (event.event.type === "mousemove") {
          eventType = 'Hover';
        } else {
          eventType = 'Other';
        }

        this.props.onChange({
          eventType: eventType,
          name: event.points[0].data.name,
          text: event.points[0].text,
          curveNumber: event.points[0].curveNumber,
          x: event.points[0].x,
          y: event.points[0].y
        });
      } // Zoom
      else if (event['xaxis.range[0]'] || event['xaxis.range']) {
          this.setState({
            xStart: event['xaxis.range[0]'] || event['xaxis.range'][0],
            xEnd: event['xaxis.range[1]'] || event['xaxis.range'][1]
          });
          this.props.onChange({
            eventType: 'Zoom',
            xStart: event['xaxis.range[0]'] || event['xaxis.range'][0],
            xEnd: event['xaxis.range[1]'] || event['xaxis.range'][1]
          });
        } // Autozoom
        else if (event['xaxis.autorange'] === true) {
            this.setState({
              xStart: null,
              xEnd: null
            });
            this.props.onChange({
              eventType: 'Autoscale'
            });
          } // Guard
          else {
              this.props.onChange(event);
            }
    }
  }, {
    key: "getData",
    // Fetch data
    value: function getData() {
      var _this$props = this.props,
          inputData = _this$props.data,
          padding = _this$props.padding,
          colorscale = _this$props.colorscale,
          backgroundcolor = _this$props.backgroundcolor; // OncoPrint equivalent of x, y

      var events = (0, _utils.aggregate)(inputData);
      var genes = (0, _utils.getSortedGenes)(inputData);
      var samples = (0, _utils.getSortedSamples)(inputData);
      var ratios = (0, _utils.getEventRatiosPerGene)(inputData, samples.length);

      var formatGenes = function formatGenes(list) {
        return list.map(function (gene) {
          return "".concat(gene, " (").concat(ratios[gene], "%)");
        });
      };

      var base = 0;
      var bBackground = [];
      var tBackground = [];
      var xBackground = [];
      var yBackground = []; // Background is used to draw the matrix (genes * samples)

      samples.forEach(function (s) {
        bBackground.push.apply(bBackground, _toConsumableArray(Array(genes.length).fill(base)));
        tBackground.push.apply(tBackground, _toConsumableArray(Array(genes.length).fill(s)));
        xBackground.push.apply(xBackground, _toConsumableArray(Array(genes.length).fill(1)));
        yBackground.push.apply(yBackground, _toConsumableArray(formatGenes(genes)));
        base += 1;
      });
      var background = {
        base: bBackground.map(function (i) {
          return i + padding;
        }),
        hoverinfo: 'text',
        marker: {
          color: backgroundcolor
        },
        name: 'No alteration',
        text: tBackground,
        orientation: 'h',
        type: 'bar',
        x: xBackground.map(function (i) {
          return i - padding * 2;
        }),
        y: yBackground
      };
      var data = [background];
      Object.keys(events).forEach(function (key) {
        var aggr = events[key]; // Resize width depending on the mutation type

        var width = 0.4;

        if (aggr.type === 'CNA') {
          width = 0.8;
        } else if (aggr.type === 'EXP') {
          width = 0.6;
        } // Mutations should have the original text on it, not the type of mutation


        var text_arr = aggr.events.map(function (event) {
          return "".concat(event.sample, "<br>").concat((0, _utils.getDisplayName)(event));
        }); // where to draw a bar for this entry

        var indexes = aggr.events.map(function (e) {
          return e.sample;
        }).map(function (s) {
          return samples.findIndex(function (sample) {
            return sample === s;
          });
        });
        data.push({
          base: indexes.map(function (i) {
            return i + padding;
          }),
          hoverinfo: 'text',
          marker: {
            color: (0, _utils.getColor)(aggr.events[0], colorscale)
          },
          name: (0, _utils.getDisplayName)(aggr.events[0]),
          text: text_arr,
          orientation: 'h',
          type: 'bar',
          width: width,
          x: Array(aggr.events.length).fill(1).map(function (i) {
            return i - padding * 2;
          }),
          y: formatGenes((0, _utils.getGeneNames)(aggr.events))
        });
      });
      return data;
    } // Fetch layout

  }, {
    key: "getLayout",
    value: function getLayout() {
      var _this$props2 = this.props,
          showlegend = _this$props2.showlegend,
          showoverview = _this$props2.showoverview,
          width = _this$props2.width,
          height = _this$props2.height;
      var _this$state = this.state,
          xStart = _this$state.xStart,
          xEnd = _this$state.xEnd; // Get initial range

      var initialRange = [xStart, xEnd];
      var layout = {
        barmode: 'stack',
        hovermode: 'closest',
        showlegend: showlegend,
        xaxis: {
          showgrid: false,
          showticklabels: false,
          zeroline: false,
          range: initialRange,
          automargin: true
        },
        yaxis: {
          showgrid: false,
          zeroline: false,
          fixedrange: true,
          automargin: true
        },
        margin: {
          t: 20,
          r: 20,
          b: 20
        }
      };

      if (showoverview) {
        layout.xaxis.rangeslider = {
          autorange: true
        };
      }

      return {
        layout: layout,
        width: width,
        height: height
      };
    } // Set xStart and xEnd on load

  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this$resetWindowing = this.resetWindowing(this.props),
          xStart = _this$resetWindowing.xStart,
          xEnd = _this$resetWindowing.xEnd;

      this.setState({
        xStart: xStart,
        xEnd: xEnd
      });
    } // Reset xStart and xEnd on data change

  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(prevProps, prevState) {
      if (this.props.data !== prevProps.data) {
        var _this$resetWindowing2 = this.resetWindowing(this.props),
            xStart = _this$resetWindowing2.xStart,
            xEnd = _this$resetWindowing2.xEnd;

        this.setState({
          xStart: xStart,
          xEnd: xEnd
        });
      }
    } // Main

  }, {
    key: "render",
    value: function render() {
      var data = this.getData();

      var _this$getLayout = this.getLayout(),
          layout = _this$getLayout.layout,
          width = _this$getLayout.width,
          height = _this$getLayout.height;

      var other = {
        style: {
          width: width,
          height: height
        },
        useResizeHandler: true
      };
      return _react.default.createElement("div", null, _react.default.createElement(_reactPlotly.default, _extends({
        data: data,
        layout: layout,
        onClick: this.handleChange,
        onHover: this.handleChange,
        onRelayout: this.handleChange
      }, other)));
    }
  }]);

  return OncoPrint;
}(_react.PureComponent);

exports.default = OncoPrint;
OncoPrint.propTypes = {
  /**
   * Input data, in CBioPortal format where each list entry is a dict
   * consisting of 'sample', 'gene', 'alteration', and 'type'
   */
  data: _propTypes.default.array,
  // TODO: Add remove empty columns prop

  /**
   * Adjusts the padding (as a proportion of whitespace) between two tracks.
   * Value is a ratio between 0 and 1.
   * Defaults to 0.05 (e.g. 5%). If set to 0, plot will look like a heatmap.
   */
  padding: _propTypes.default.number,

  /**
   * If not null, will override the default OncoPrint colorscale.
   * Default OncoPrint colorscale same as CBioPortal implementation.
   * Make your own colrscale as a {'mutation': COLOR} dict.
   * Supported mutation keys are ['MISSENSE, 'INFRAME', 'FUSION',
   * 'AMP', 'GAIN', 'HETLOSS', 'HMODEL', 'UP', 'DOWN']
   * Note that this is NOT a standard plotly colorscale.
   */
  colorscale: _propTypes.default.oneOfType([_propTypes.default.bool, _propTypes.default.object]),

  /**
   * Default color for the tracks, in common name, hex, rgb or rgba format.
   * If left blank, will default to a light grey rgb(190, 190, 190).
   */
  backgroundcolor: _propTypes.default.string,

  /**
   *.Toogles whether or not to show a legend on the right side of the plot,
   * with mutation information.
   */
  range: _propTypes.default.array,

  /**
   *.Toogles whether or not to show a legend on the right side of the plot,
   * with mutation information.
   */
  showlegend: _propTypes.default.bool,

  /**
   *.Toogles whether or not to show a heatmap overview of the tracks.
   */
  showoverview: _propTypes.default.bool,

  /**
   * Width of the OncoPrint.
   * Will disable auto-resizing of plots if set.
   */
  width: _propTypes.default.oneOfType([_propTypes.default.number, _propTypes.default.string]),

  /**
   * Height of the OncoPrint.
   * Will disable auto-resizing of plots if set.
   */
  height: _propTypes.default.oneOfType([_propTypes.default.number, _propTypes.default.string])
};
OncoPrint.defaultProps = {
  // Data
  padding: 0.05,
  colorscale: null,
  backgroundcolor: 'rgb(190, 190, 190)',
  // Layout
  range: [null, null],
  showlegend: true,
  showoverview: true,
  // Other
  width: null,
  height: 500
};